#
# Ubuntu PractiTest Community results processor
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""Script to allow uploading of data to a ubuntu-pt-community service."""

# For example if you're running the runserver.py script:
# python3 upload_results.py <path to json file> http://localhost:5000/v1/upload

import argparse
import json
import requests
import getpass

from oauthlib import oauth1


class TwoFactorRequiredError(RuntimeError):
    """Second factor authentication required."""


class InvalidCredentialsError(RuntimeError):
    """Username or password incorrect for account."""


def parse_args():
    arg_parser = argparse.ArgumentParser()
    arg_parser.add_argument('file', help='Path to the json file to upload.')
    arg_parser.add_argument(
        'url',
        # Note this should be https
        help='url to the service to upload to. Must be in the form "http://..'
    )
    arg_parser.add_argument(
        '--email',
        help='Email address to use for logging in. If not provided you will be prompted for it.'  # NOQA
    )
    arg_parser.add_argument(
        '--staging',
        help='If set use staging login server. (Only useful for testing.)',
        action='store_true'
    )
    return arg_parser.parse_args()


def attempt_ubuntu_sso_login(email_address, password, staging=False, otp=None):
    """Obtain OAuth token that can be used to authenticate with transport
    end-points.

    :param email_address: email address for user account.
    :param password: password for user account
    :param otp: one-time-password to use (if required, i.e. previous call
      raised `TwoFactorRequiredError`.)

    :raises TwoFactorRequired: when the one time password `otp` is required to
      log in.

    """
    token_name = 'ubuntu-pt-community'
    post_data = dict(
        email=email_address,
        password=password,
        token_name=token_name
    )

    if otp is not None:
        post_data['otp'] = otp

    json_headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
    }

    sso_base_url = 'https://login.ubuntu.com/api/v2'
    if staging:
        sso_base_url = 'https://login.staging.ubuntu.com/api/v2'
    sso_url = '{}/tokens/oauth'.format(sso_base_url)

    response = requests.post(
        sso_url,
        json.dumps(post_data),
        headers=json_headers
    )

    response_json = response.json()

    if not response.ok:
        if response.status_code == 401:
            if response_json.get('code') == 'TWOFACTOR_REQUIRED':
                raise TwoFactorRequiredError()
            elif response_json.get('code') == 'INVALID_CREDENTIALS':
                raise InvalidCredentialsError()
        raise ValueError(response.text)

    return response_json


def get_oauth_signed_request(url, oauth_credentials, method='POST'):
    """Return details for a signed request to `url` using oauth details in
    `oauth_credentials`.

    :param url: Url for which the signed request will be for.
    :param oauth_credentials: Dictionary containing OAuth credentials to use
      for signing the request.
      Must contain: consumer_key, consumer_secret, token_key, token_secret.
    :param method: Method type that will be made to the `url` (i.e. POST, GET)
    :returns: Dictionary containing details relevent for a signed header.
    :raises KeyError: If lacking any of the required oauth_credential details.

    """
    client = oauth1.Client(
        client_key=oauth_credentials['consumer_key'],
        client_secret=oauth_credentials['consumer_secret'],
        resource_owner_key=oauth_credentials['token_key'],
        resource_owner_secret=oauth_credentials['token_secret'],
        signature_method=oauth1.SIGNATURE_HMAC,
        realm='Checkbox uploading',
    )

    uri, headers, body = client.sign(url, method)
    return headers


def get_upload_details():
    args = parse_args()
    return (args.file, args.url, args.email, args.staging)


if __name__ == '__main__':
    file_path, url, email, use_staging = get_upload_details()

    if use_staging:
        print("Using staging server")

    if email is None:
        email = input('Email address: ')

    password = getpass.getpass('Password: ')

    try:
        oauth_creds = attempt_ubuntu_sso_login(email, password, use_staging)
    except TwoFactorRequiredError:
        otp = input('Your next 2fa code: ').strip()
        oauth_creds = attempt_ubuntu_sso_login(
            email,
            password,
            use_staging,
            otp=otp
        )
    except InvalidCredentialsError:
        print('ERROR: Invalid credentials')

    headers = get_oauth_signed_request(url, oauth_creds)
    form_data = dict(uploader_email=email)

    print('Uploading file to service.')
    with open(file_path, 'rb') as f:
        upload_data = dict(data=f)
        # Right, now we can post the details.
        response = requests.post(
            url,
            files=upload_data,
            data=form_data,
            headers=headers
        )

    if not response.ok:
        print('Upload failed: <{code}> {msg}'.format(
            code=response.status_code,
            msg=response.reason)
        )
    else:
        print('Successful: {}'.format(response.text))
