#
# Ubuntu PractiTest Community results processor
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import datetime
import json
import logging
import os
import pymongo
from flask import request

from qakit.practitest import practitest
from qakit.practitest.report_results_to_practitest import _read_config
from qakit.practitest.report_checkbox_results_to_practitest import (
    upload_results
)
from ubuntu_pt_community import auth
from ubuntu_pt_community.db import get_results_collection


logger = logging.getLogger(__name__)

api_preamble = 'v1'


def define_routes(webapp):
    webapp.add_url_rule(
        '/{}/upload'.format(api_preamble),
        view_func=upload,
        methods=['POST']
    )


def upload():
    # Steps:
    #  - Authenticate request (i.e. do we know this person and care?)
    #  - Grab uploaded data
    #  - Keep local (raw) copy?
    #  - Upload to PT

    auth.validate_request(request)

    try:
        upload_json_data = request.files['data'].stream.getvalue()
    except KeyError:
        err_message = 'No data files provided.'
        logger.error(err_message)
        return (err_message, 500, {})

    issues = []
    try:
        logger.info('Attempting to store upload results.')
        store_upload_details(request, upload_json_data)
    except Exception as e:
        err_message = 'Failed to store upload details: {}'.format(str(e))
        logger.error(err_message)
        issues.append(err_message)

    try:
        logger.info('Attempting to upload results to practitest.')
        upload_to_practitest(upload_json_data)
    except Exception as e:
        err_message = 'Failed to upload data to practitest: {}'.format(str(e))
        logger.error(err_message)
        issues.append(err_message)

    if issues:
        success_string = 'Uploaded with some issues: {}'.format(
            '\n'.join(issues)
        )
    else:
        success_string = 'Uploaded with no issues.'
    return success_string


def upload_to_practitest(json_data):
    kwargs = _read_config(os.path.join(
        os.environ['CONFIG_DIR'], 'config.ini'
    ))
    kwargs['proxy_dict'] = {
        'https': 'https://squid.internal:3128'
    }
    pt = practitest.PractitestSession(**kwargs)
    upload_results(json.loads(json_data.decode('utf-8')), pt)


def store_upload_details(request, json_data):
    logger.info('Storing upload details.')
    # If email address is None, should we continue to store the data?
    email_address = get_user_email_address(request)
    if email_address is None:
        logger.warning('Storing results against no email address.')

    details = dict(
        user_email=email_address,
        uploaded=datetime.datetime.utcnow(),
        results=json_data
    )

    try:
        collection = get_results_collection()
        insert_id = collection.insert(details)
        logger.info(
            'Inserted details for {} with id {}'.format(
                email_address,
                insert_id
            )
        )
    except pymongo.errors.DuplicateKeyError:
        # This shouldn't happen due to the timestamp.
        logger.error('Attempting to store duplicate results')
    except pymongo.errors.ConnectionFailure:
        logger.error('Failed to connect to database. Results not stored.')
    except Exception as e:
        logger.error('Failed to get database connection: ', e)


def get_user_email_address(request):
    try:
        return request.form['uploader_email']
    except KeyError:
        logger.error('Unable to extract email address from header.')
        return None
