#
# Ubuntu PractiTest Community results processor

# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import json
import logging
from collections import (
    defaultdict,
    namedtuple,
)
from flask import render_template

from ubuntu_pt_community import db

logger = logging.getLogger(__name__)


def view_all_results():
    """Render report displaying all uploaded testsuites with stats.

    Stats displayed are number of runs, number of passes and fails and the % of
    successful runs.
    """
    results_collection = db.get_results_collection()
    all_uploads = results_collection.find()
    simple_report_data = _get_simple_report_data(all_uploads)
    return render_template(
        'results/all_testsuites.html',
        results=simple_report_data
    )


def _get_simple_report_data(all_uploads):
    """Produce a list of suite results.

    :param all_uploads: list of dicts containing uploaded data details.
    :returns: a list of TestsuiteResult objects.
    """
    TestsuiteResult = namedtuple(
        'TestsuiteResult',
        ['name', 'runs', 'passes', 'fails', 'success_rate']
    )

    # all_uploads will be a list of dicts
    # dict will have keys:
    #   - results: a json string with the result details.
    #   - user_email: email of the user who uploaded the results
    #   - uploaded: a date of uploading
    testsuites = defaultdict(list)
    for upload in all_uploads:
        upload_id = str(upload['_id'])

        try:
            results = _result_dict_from_document(upload)
        except KeyError as e:
            logger.warning(e)
            continue

        only_tests = _get_only_testcases(results)

        for testname in only_tests:
            # outcome will be 'pass' or 'fail'
            try:
                testsuites[testname].append(only_tests[testname]['outcome'])
            except KeyError:
                logger.error(
                    'Testsuite has not outcome. Document ID: ',
                    upload_id
                )

    results = []
    for suite in testsuites:
        runs = len(testsuites[suite])
        passes = len([t for t in testsuites[suite] if t == 'pass'])
        fails = runs - passes
        success_rate = format(passes / runs * 100, '.2f')

        results.append(
            TestsuiteResult(suite, runs, passes, fails, success_rate)
        )
    return results


def view_latest_uploads():
    """Render report displaying a list of uploaded results, latest first."""
    results_collection = db.get_results_collection()
    # Sorted by latest first. . .
    all_uploads = results_collection.find().sort('uploaded', -1)

    # slight shim over the returned data to prepare it for presention.
    sanitised_uploads = []
    for upload in all_uploads:
        upload_date = upload['uploaded'].strftime('%Y-%b-%d %H:%M:%S')
        uploader_email = upload.get('user_email') or 'Anonymous'

        try:
            results = _result_dict_from_document(upload)
        except KeyError as e:
            logger.warning(e)
            continue

        testsuite_details = _get_only_testcases(results)
        uploaded_testsuite_names = testsuite_details.keys()

        sanitised_uploads.append(
            dict(
                upload_date=upload_date,
                testsuites=uploaded_testsuite_names,
                uploader_email=uploader_email
            )
        )

    return render_template(
        'results/latest_uploads.html',
        all_uploads=sanitised_uploads
    )


def _result_dict_from_document(upload):
    """Return a dict of result details.

    Parses the json result string and constructs a dict containing the details.

    :param upload: dict containing upload details. Must contain the key
      'results' to be successful.
    :raises KeyError: if no 'results' are found in the upload dict.
    """
    try:
        results_json = upload['results']
        return json.loads(results_json.decode('utf-8'))
    except KeyError:
        upload_id = str(upload.get('_id'), 'No ID.')
        logger.warning(
            'Skipping document. No results or result_map available',
            upload_id,
        )


def _get_only_testcases(upload_details):
    """Return details only for testsuites.

    Uploaded results can contain detials that aren't testsuite related
    (i.e. system details tests were run on.) Extract only the details that are
    for testsuitse.

    :param uploaded_details: dict containing keys 'resource_map' and
    'result_map'.
    """
    removal_keys = upload_details['resource_map'].keys()

    return {k: v for k, v in upload_details['result_map'].items()
            if k not in removal_keys}
